=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2012-03-28 17:04:06 +0000
+++ bzrlib/tests/test_progress.py	2012-04-30 10:44:04 +0000
@@ -17,16 +17,18 @@
 
 from cStringIO import StringIO
 
+from bzrlib import (
+    tests,
+    )
 from bzrlib.progress import (
     ProgressTask,
     )
-from bzrlib.tests import TestCase
 from bzrlib.ui.text import (
     TextProgressView,
     )
 
 
-class TestTextProgressView(TestCase):
+class TestTextProgressView(tests.TestCase):
     """Tests for text display of progress bars.
 
     These try to exercise the progressview independently of its construction,
@@ -35,13 +37,16 @@
     # The ProgressTask now connects directly to the ProgressView, so we can
     # check them independently of the factory or of the determination of what
     # view to use.
-    
+
+    def make_view_only(self, out, width=79):
+        view = TextProgressView(out)
+        view._avail_width = lambda: width
+        return view
+
     def make_view(self):
         out = StringIO()
-        view = TextProgressView(out, "utf-8")
-        view._avail_width = lambda: 79
-        return out, view
-    
+        return out, self.make_view_only(out)
+
     def make_task(self, parent_task, view, msg, curr, total):
         # would normally be done by UIFactory; is done here so that we don't
         # have to have one.
@@ -155,10 +160,29 @@
            line) 
         self.assertEqual(len(line), 79)
 
-    def test_render_progress_unicode(self):
-        out, view = self.make_view()
-        task = self.make_task(None, view, u"\xa7", 0, 1)
-        view.show_progress(task)
-        self.assertEqual(
-'\r/ \xc2\xa7 0/1                                                                       \r',
+    def test_render_progress_unicode_enc_utf8(self):
+        out = tests.StringIOWrapper()
+        out.encoding = "utf-8"
+        view = self.make_view_only(out, 20)
+        task = self.make_task(None, view, u"\xa7", 0, 1)
+        view.show_progress(task)
+        self.assertEqual('\r/ \xc2\xa7 0/1            \r',
+            out.getvalue())
+
+    def test_render_progress_unicode_enc_missing(self):
+        out = StringIO()
+        self.assertRaises(AttributeError, getattr, out, "encoding")
+        view = self.make_view_only(out, 20)
+        task = self.make_task(None, view, u"\xa7", 0, 1)
+        view.show_progress(task)
+        self.assertEqual('\r/ ? 0/1             \r',
+            out.getvalue())
+
+    def test_render_progress_unicode_enc_none(self):
+        out = tests.StringIOWrapper()
+        out.encoding = None
+        view = self.make_view_only(out, 20)
+        task = self.make_task(None, view, u"\xa7", 0, 1)
+        view.show_progress(task)
+        self.assertEqual('\r/ ? 0/1             \r',
             out.getvalue())

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2012-03-28 17:04:06 +0000
+++ bzrlib/ui/text.py	2012-04-30 10:44:04 +0000
@@ -407,7 +407,7 @@
     def __init__(self, term_file, encoding=None, errors="replace"):
         self._term_file = term_file
         if encoding is None:
-            self._encoding = getattr(term_file, "encoding", "ascii")
+            self._encoding = getattr(term_file, "encoding", None) or "ascii"
         else:
             self._encoding = encoding
         self._encoding_errors = errors

